@@ -8,18 +8,18 @@ use File::Path qw( make_path );
 use Cwd 'abs_path';
 require 5.008001; # Supporting 5.8.1+
 
-our $VERSION = '0.001006'; # 0.1.6
+our $VERSION = '0.001007'; # 0.1.7
 $VERSION = eval $VERSION;
 
 my @accessors = qw(
     pid color_map name program program_args directory quiet
-    uid path gid scan_name stdout_file stderr_file pid_file fork data
+    path scan_name stdout_file stderr_file pid_file fork data
     lsb_start lsb_stop lsb_sdesc lsb_desc redirect_before_fork init_config
     kill_timeout umask resource_dir help init_code
     prereq_no_process foreground reload_signal stop_signals
 );
 
-my $cmd_opt = "[start|stop|restart|reload|status|show_warnings|get_init_file|help]";
+my $cmd_opt = "[start|stop|restart|reload|status|foreground|show_warnings|get_init_file|help]";
 
 # Accessor building
 
@@ -40,27 +40,47 @@ for my $method ( @accessors ) {
 # so I'm making my own triggers for user and group.
 
 sub user {
-    my ( $self, $user ) = @_;
+    my $self = shift;
 
-    if ( $user ) {
-        $self->{user} = $user;
-        $self->_set_uid_from_name( $user );
+    if ( @_ ) {
+        $self->{user} = shift;
+        delete $self->{uid};
     }
 
     return $self->{user};
 }
 
 sub group {
-    my ( $self, $group ) = @_;
+    my $self = shift;
 
-    if ( $group ) {
-        $self->{group} = $group;
-        $self->_set_gid_from_name( $group );
+    if ( @_ ) {
+        $self->{group} = shift;
+        delete $self->{gid};
     }
 
     return $self->{group};
 }
 
+sub uid {
+    my $self = shift;
+
+    return $self->{uid} = shift if @_;
+
+    $self->_set_uid_from_name unless exists $self->{uid};
+
+    return $self->{uid}
+}
+
+sub gid {
+    my $self = shift;
+
+    return $self->{gid} = shift if @_;
+
+    $self->_set_gid_from_name unless exists $self->{gid};
+
+    return $self->{gid}
+}
+
 sub new {
     my ( $class, @in ) = @_;
 
@@ -78,16 +98,12 @@ sub new {
         stop_signals            => [ qw(TERM TERM INT KILL) ],
     }, $class;
 
-    for my $accessor ( @accessors ) {
+    for my $accessor ( @accessors, qw(uid gid user group) ) {
         if ( exists $args->{$accessor} ) {
             $self->{$accessor} = delete $args->{$accessor};
         }
     }
 
-    # Set the user/groups.
-    $self->user(delete $args->{user}) if exists $args->{user};
-    $self->group(delete $args->{group}) if exists $args->{group};
-
     # Shortcut caused by setting foreground or using the ENV to do it.
     if ( ( $self->foreground == 1 ) || ( $ENV{DC_FOREGROUND} ) ) {
         $self->fork( 0 );
@@ -100,21 +116,62 @@ sub new {
     return $self;
 }
 
+sub with_plugins {
+    my ( $class, @in ) = @_;
 
-# Set the uid, triggered from setting a user string.
+    # ->with_plugins()->new is just ->new...
+    return $class unless @in;
+
+    # Make sure we have Role::Tiny installed.
+    local $@;
+    eval "require Role::Tiny";
+    if ( $@ ) {
+        die "Error: Role::Tiny is required for with_plugins to function.\n";
+    }
+
+    # Take an array or arrayref as an argument
+    # and mutate it into a list like this:
+    #   'Module'  -> Becomes -> 'Root::Module'
+    #   '+Module' ->  Becomes -> 'Module'
+    my @plugins = map {
+        substr( $_, 0, 1 ) eq '+'
+            ? substr( $_, 1 )
+            : "Daemon::Control::Plugin::$_" 
+    } ref $in[0] eq 'ARRAY' ? @{ $in[0] } : @in;
+
+
+    # Compose the plugins into our class, and return for the user
+    # to call ->new().
+    return Role::Tiny->create_class_with_roles( $class, @plugins );
+}
+
+# Set the uid, triggered from getting the uid if the user has changed.
 sub _set_uid_from_name {
-    my ( $self, $name ) = @_;
-    my $uid = getpwnam( $name );
+    my ( $self ) = @_;
+    return unless defined $self->user;
+
+    my $uid = getpwnam( $self->user );
     die "Error: Couldn't get uid for non-existent user " . $self->user
         unless defined $uid;
     $self->trace( "Set UID => $uid" );
     $self->uid( $uid );
 }
 
-# Set the uid, triggered from setting a group string.
+# Set the uid, triggered from getting the gid if the group has changed.
 sub _set_gid_from_name {
-    my ( $self, $name ) = @_;
-    my $gid = getgrnam( $name );
+    my ( $self ) = @_;
+
+    # Grab the GID if we have a UID but no GID.
+    if ( !defined $self->group && defined $self->uid ) {
+        my ( $gid ) = ( (getpwuid( $self->uid ))[3] );
+        $self->gid( $gid );
+        $self->trace( "Implicit GID => $gid" );
+        return $gid;
+    }
+
+    return unless defined $self->group;
+
+    my $gid = getgrnam( $self->group );
     die "Error: Couldn't get gid for non-existent group " . $self->group
         unless defined $gid;
     $self->trace( "Set GID => $gid" );
@@ -599,13 +656,6 @@ sub run_command {
         die "Error: name must be defined.";
     }
 
-    # Grab the GID if we have a UID but no GID.
-    if ( $self->uid && ! $self->gid ) {
-        my ( $gid ) = ( (getpwuid( $self->uid ))[3] );
-        $self->gid( $gid );
-        $self->trace( "Implicit GID => $gid" );
-    }
-    
     my $called_with = $arg || "help";
     $called_with =~ s/^[-]+//g; # Allow people to do --command too.
 
@@ -625,7 +675,7 @@ sub run_command {
 
 # Application Code.
 sub run {
-    shift->run_command( @ARGV );
+    exit shift->run_command( @ARGV );
 }
 
 sub trace {
@@ -916,7 +966,7 @@ and do all that fun stuff.  This mode is recommended when the program you want
 to control has its own daemonizing code.  It is important to note that the PID
 file should be set to whatever PID file is used by the daemon.
 
-In no-fork mode, <Cfork(0)>, the program is run in the foreground.  By default
+In no-fork mode, C<fork(0)>, the program is run in the foreground.  By default
 quiet is still turned off, so status updates will be shown on the screen such
 as that the daemon started.  A shortcut to turn status off and go into foreground
 mode is C<foreground> being set to 1, or C<DC_FOREGROUND> being set as an
@@ -997,6 +1047,40 @@ stopping the daemon.
 Default signals are C<TERM>, C<TERM>, C<INT> and C<KILL> (yes, C<TERM>
 is tried twice).
 
+=head1 PLUGINS
+
+Daemon Control supports a simple plugin system using L<Role::Tiny>.
+
+=head2 with_plugins
+
+With plugins adds the plugins to Daemon::Control.
+
+    Daemon::Control->with_plugins( qw( MyFirstPlugin +MySecondPlugin) )->new(
+    ...
+    );
+
+Note:
+
+MyFirstPlugin will load Daemon::Control::Plugin::MyFirstPlugin
+
++MySecondPlugin will load MySecondPlugin
+
+
+=head2 Writing A Plugin
+
+Your plugin should use the name Daemon::Control::Plugin::YourModuleName and
+YourModuleName should reasonably match the effect your plugin has on
+Daemon::Control.
+
+You can replace Daemon::Control methods by writing your own and using 
+Role::Tiny within your class to allow it to be composed into Daemon::Control.
+
+The default Daemon::Control ships with no dependancies and supports Perl 
+5.8.1+, to use the plugin system your module MUST declare dependency on 
+L<Role::Tiny> and if you wish to use the C<around>, C<before> and C<after> 
+your module MUST declare dependance on L<Class::Method::Modifiers> in your 
+package.
+
 =head1 METHODS
 
 =head2 run_command
@@ -1110,6 +1194,10 @@ Kaitlyn Parkhurst (SymKat) I<E<lt>symkat@symkat.comE<gt>> ( Blog: L<http://symka
 
 =item * Ævar Arnfjörð Bjarmason (avar) I<E<lt>avar@cpan.orgE<gt>>
 
+=item * Kieren Diment I<E<lt>zarquon@cpan.org<gt>>
+
+=item * Mark Curtis I<E<lt>mark.curtis@affinitylive.com<gt>>
+
 =back
 
 =head2 SPONSORS
